@@ -1,6 +1,6 @@
 use std::collections::HashMap;
 
-use crate::{derive::KeyDescs, KeyDesc, PropertyError};
+use crate::{derive::KeyDescs, KeyDesc, PropertyError, Res};
 
 /// Application info.
 #[derive(Debug)]
@@ -30,7 +30,7 @@ macro_rules! app_info {
     };
 }
 
-fn parse(s: String) -> Result<(String, String), PropertyError> {
+fn parse(s: String) -> Res<(String, String)> {
     if let Some(usize) = s.find("=") {
         return Ok((s[..usize - 1].to_string(), s[usize..].to_string()));
     }
@@ -39,10 +39,7 @@ fn parse(s: String) -> Result<(String, String), PropertyError> {
 
 #[cfg_attr(docsrs, doc(cfg(feature = "args")))]
 /// Generate source from args.
-pub(crate) fn from_args(
-    desc: Vec<KeyDesc>,
-    info: AppInfo<'_>,
-) -> Result<HashMap<String, String>, PropertyError> {
+pub(crate) fn from_args(desc: Vec<KeyDesc>, info: AppInfo<'_>) -> Res<HashMap<String, String>> {
     let help = format!("KEYS:\n{}\n", &KeyDescs(desc));
 
     let mut app = clap::App::new(info.name)
@@ -68,7 +65,7 @@ pub(crate) fn from_args(
         .unwrap_or(vec![])
         .into_iter()
         .map(|f| parse(f))
-        .collect::<Result<Vec<(String, String)>, PropertyError>>()?
+        .collect::<Res<Vec<(String, String)>>>()?
         .into_iter()
         .collect::<HashMap<String, String>>())
 }
