@@ -7,6 +7,8 @@
 //!    * [Key Convention](#key-convention)
 //!    * [Value Placeholder Parsing](#value-placeholder-parsing)
 //!    * [Attributes For Derive](#attributes-for-derive)
+//!    * [Reload Configuration](#reload-configuration)
+//!    * [Resource Factory](#resource-factory)
 //!
 //! ## About
 //! `salak` is a multi layered configuration loader with many predefined sources. Also it
@@ -61,8 +63,8 @@
 //! 2. Custom arguments source. [`SalakBuilder::set()`] can set a single kv,
 //! and [`SalakBuilder::set_args()`] can set a group of kvs.
 //! 3. System environment source. Implemented by [`source::system_environment`].
-//! 4. Profile specified file source, eg. `app-dev.toml`
-//! 5. Not profile file source, eg. `app.toml`
+//! 4. Profile specified file source, eg. `app-dev.toml`, supports reloading.
+//! 5. No profile file source, eg. `app.toml`, supports reloading.
 //! 6. Custom sources, which can register by [`Salak::register()`].
 //!
 //! #### Key Convention
@@ -98,6 +100,16 @@
 //!    * `#[salak(name = "key")]`, this attr can specify property key, default convension is use field name.
 //!    * `#[salak(desc = "Field Description")]`, this attr can be describe this property.
 //!
+//! #### Reload Configuration
+//! `salak` supports reload configurations. Since in rust mutable
+//! and alias can't be used together, here we introduce a wrapper
+//! [`wrapper::IORef`] for updating values when reloading.
+//!
+//! #### Resource Factory
+//! [`Resource`] defines a standard way to create instance. [`Factory`] provides functions to initialize resource
+//! and cache resource. Please refer to [salak_factory](https://docs.rs/salak_factory) for resource usage.
+//! Feature 'app' should be open for this feature.
+//!
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![warn(
     anonymous_parameters,
@@ -148,11 +160,11 @@ mod raw_ioref;
 mod raw_vec;
 use crate::env::PREFIX;
 pub use crate::env::{Salak, SalakBuilder};
-mod enums;
 mod env;
+mod raw_enum;
 
-pub use crate::enums::EnumProperty;
 pub use crate::err::PropertyError;
+pub use crate::raw_enum::EnumProperty;
 
 mod source_map;
 #[cfg(feature = "rand")]
@@ -169,17 +181,29 @@ mod source_yaml;
 use crate::source::Key;
 use crate::source::SubKeys;
 
+#[cfg(feature = "app")]
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+mod app;
+#[cfg(feature = "app")]
+#[cfg_attr(docsrs, doc(cfg(feature = "app")))]
+pub use crate::app::*;
+
 #[cfg(test)]
 #[macro_use(quickcheck)]
 extern crate quickcheck_macros;
 
 /// Salak wrapper for configuration parsing.
+///
+/// Wrapper can determine extra behavior for parsing.
+/// Such as check empty of vec or update when reloading.
 pub mod wrapper {
     pub use crate::raw_ioref::IORef;
     pub use crate::raw_vec::NonEmptyVec;
 }
 
-/// Salak property sources.
+/// Salak sources.
+///
+/// This mod exports all pub sources.
 pub mod source {
 
     #[cfg(feature = "args")]
@@ -191,43 +215,73 @@ pub mod source {
     pub use crate::source_map::HashMapSource;
 }
 
-/// An abstract source loader from various sources,
-/// such as command line arguments, system environment, files, etc.
+pub(crate) type Res<T> = Result<T, PropertyError>;
+pub(crate) type Void = Res<()>;
+
+/// A property source defines how to load properties.
+/// `salak` has some predefined sources, user can
+/// provide custom source by implementing this trait.
+///
+/// Sources provided by `salak`.
+///
+/// * hashmap source
+/// * std::env source
+/// * toml source
+/// * yaml source
 pub trait PropertySource: Send + Sync {
     /// [`PropertySource`] name.
     fn name(&self) -> &str;
 
-    /// Get property by name.
+    /// Get property by key.
     fn get_property(&self, key: &Key<'_>) -> Option<Property<'_>>;
 
-    /// Return next sub keys with prefix, sub keys are seperated by dot(.) in a key.
+    /// Get all subkeys with given key.
+    ///
+    /// Subkeys are keys without dot('.').
+    /// This method is unstable, and will be simplified by hidding
+    /// Key and SubKeys.
     fn get_sub_keys<'a>(&'a self, key: &Key<'_>, sub_keys: &mut SubKeys<'a>);
 
     /// Check whether the [`PropertySource`] is empty.
-    /// Empty source will not be ignored when register to registry.
+    /// Empty source will be ignored when registering to `salak`.
     fn is_empty(&self) -> bool;
 
-    /// Reload property source.
-    /// If nothing changes, then return none.
+    /// Reload source, if nothing changes, then return none.
     #[inline]
-    fn reload_source(&self) -> Result<Option<Box<dyn PropertySource>>, PropertyError> {
+    fn reload_source(&self) -> Res<Option<Box<dyn PropertySource>>> {
         Ok(None)
     }
 }
 
-/// An environment for getting properties with mutiple [`PropertySource`]s, placeholder resolve and other features.
+/// Environment defines interface for getting values, and reloading
+/// configurations.
+///
+/// The implementor of this trait is [`Salak`].
 pub trait Environment {
-    /// Get config with specific type.
-    fn require<T: FromEnvironment>(&self, key: &str) -> Result<T, PropertyError>;
+    /// Get value by key.
+    /// * `key` - Configuration key.
+    ///
+    /// Require means is if the value `T` is not found,
+    /// then error will be returned. But if you try to get
+    /// `Option<T>`, then not found will return `None`.
+    fn require<T: FromEnvironment>(&self, key: &str) -> Res<T>;
 
-    /// Reload configuration.
-    fn reload(&self) -> Result<bool, PropertyError>;
+    /// Reload configuration. If reloading is completed,
+    /// all values wrapped by [`wrapper::IORef`] will be updated.
+    ///
+    /// Currently, this feature is unstable, the returned bool
+    /// value means reloading is completed without error.
+    fn reload(&self) -> Res<bool>;
 
     #[cfg(feature = "derive")]
     #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
-    /// Get config with predefined prefix.
     #[inline]
-    fn get<T: PrefixedFromEnvironment>(&self) -> Result<T, PropertyError> {
+    /// Get value with predefined key.
+    ///
+    /// [`PrefixedFromEnvironment`] can be auto derives by
+    /// [`salak_derive::FromEnvironment`] macro. It provides
+    /// a standard key for getting value `T`.
+    fn get<T: PrefixedFromEnvironment>(&self) -> Res<T> {
         self.require::<T>(T::prefix())
     }
 }
@@ -240,7 +294,7 @@ pub struct SalakContext<'a> {
     key: &'a mut Key<'a>,
 }
 
-/// Generate config from environment by [`SalakContext`].
+/// Parsing value from environment by [`SalakContext`].
 pub trait FromEnvironment: Sized {
     /// Generate object from [`SalakContext`].
     /// * `val` - Property value can be parsed from.
@@ -263,8 +317,5 @@ pub trait FromEnvironment: Sized {
     /// }
     ///
     /// ```
-    fn from_env(
-        val: Option<Property<'_>>,
-        env: &mut SalakContext<'_>,
-    ) -> Result<Self, PropertyError>;
+    fn from_env(val: Option<Property<'_>>, env: &mut SalakContext<'_>) -> Res<Self>;
 }
