@@ -1,5 +1,13 @@
-use crate::PropertyError;
-use std::{collections::HashSet, time::Duration};
+#[cfg(feature = "derive")]
+use crate::{DescFromEnvironment, PrefixedFromEnvironment, SalakDescContext};
+use crate::{FromEnvironment, PropertyError, Res, SalakContext};
+use std::{
+    collections::HashSet,
+    ffi::OsString,
+    net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr, SocketAddrV4, SocketAddrV6},
+    path::PathBuf,
+    time::Duration,
+};
 
 /// Raw property, it is a temprory representation of property, which can be either [`&str`] or [`String`], or other values.
 #[derive(Clone, Debug)]
@@ -33,7 +41,43 @@ pub trait IsProperty: Sized {
     }
 
     /// Parse value from property.
-    fn from_property(_: Property<'_>) -> Result<Self, PropertyError>;
+    fn from_property(_: Property<'_>) -> Res<Self>;
+}
+
+impl<T: IsProperty> FromEnvironment for T {
+    #[inline]
+    fn from_env(val: Option<Property<'_>>, env: &mut SalakContext<'_>) -> Res<Self> {
+        if let Some(v) = val {
+            if !Self::is_empty(&v) {
+                return Self::from_property(v);
+            }
+        }
+        Err(PropertyError::NotFound(env.current_key().to_string()))
+    }
+}
+
+#[cfg(feature = "derive")]
+#[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
+impl<T: IsProperty> DescFromEnvironment for T {
+    #[inline]
+    fn key_desc(env: &mut SalakDescContext<'_>) {
+        env.current.ignore = false;
+        env.current.set_required(true);
+    }
+}
+
+impl IsProperty for () {
+    fn from_property(_: Property<'_>) -> Res<Self> {
+        Ok(())
+    }
+}
+
+#[cfg(feature = "derive")]
+#[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
+impl PrefixedFromEnvironment for () {
+    fn prefix() -> &'static str {
+        ""
+    }
 }
 
 #[inline]
@@ -51,7 +95,7 @@ impl IsProperty for String {
         false
     }
     #[inline]
-    fn from_property(p: Property<'_>) -> Result<Self, PropertyError> {
+    fn from_property(p: Property<'_>) -> Res<Self> {
         Ok(match p {
             Property::S(v) => v.to_string(),
             Property::O(v) => v,
@@ -63,8 +107,8 @@ impl IsProperty for String {
 }
 impl IsProperty for bool {
     #[inline]
-    fn from_property(p: Property<'_>) -> Result<Self, PropertyError> {
-        fn str_to_bool(v: &str) -> Result<bool, PropertyError> {
+    fn from_property(p: Property<'_>) -> Res<Self> {
+        fn str_to_bool(v: &str) -> Res<bool> {
             match v {
                 "yes" | "true" => Ok(true),
                 "no" | "false" => Ok(false),
@@ -84,7 +128,7 @@ macro_rules! impl_property_num {
     ($($x:ident),+) => {$(
             impl IsProperty for $x {
                 #[inline]
-                fn from_property(p: Property<'_>) -> Result<Self, PropertyError> {
+                fn from_property(p: Property<'_>) -> Res<Self> {
                     use std::convert::TryFrom;
                     Ok(match p {
                     Property::S(s) => s.parse::<$x>()?,
@@ -107,7 +151,7 @@ macro_rules! impl_property_float {
             #[allow(trivial_numeric_casts)]
             impl IsProperty for $x {
                 #[inline]
-                fn from_property(p: Property<'_>) -> Result<Self, PropertyError> {
+                fn from_property(p: Property<'_>) -> Res<Self> {
                     Ok(match p {
                     Property::S(s) => s.parse::<$x>()?,
                     Property::O(s) => s.parse::<$x>()?,
@@ -125,7 +169,7 @@ macro_rules! impl_property_float {
 impl_property_float!(f32, f64);
 
 #[inline]
-fn parse_duration_from_str(du: &str) -> Result<Duration, PropertyError> {
+fn parse_duration_from_str(du: &str) -> Res<Duration> {
     let mut i = 0;
     let mut multi = 1;
     let mut last = None;
@@ -163,7 +207,7 @@ fn parse_duration_from_str(du: &str) -> Result<Duration, PropertyError> {
 }
 
 impl IsProperty for Duration {
-    fn from_property(p: Property<'_>) -> Result<Self, PropertyError> {
+    fn from_property(p: Property<'_>) -> Res<Self> {
         match p {
             Property::O(du) => parse_duration_from_str(&du),
             Property::S(du) => parse_duration_from_str(du),
@@ -335,6 +379,34 @@ impl<'a> SubKeys<'a> {
     }
 }
 
+macro_rules! impl_property_from_str {
+    ($($x:ident),+) => {$(
+            impl IsProperty for $x {
+                #[inline]
+                fn from_property(p: Property<'_>) -> Res<Self> {
+                    use std::str::FromStr;
+                    Ok(match p {
+                    Property::S(s) => <$x>::from_str(s)?,
+                    Property::O(s) => <$x>::from_str(&s)?,
+                    _ => return Err(PropertyError::parse_fail("can not convert")),
+                    })
+                }
+
+            }
+            )+}
+}
+
+impl_property_from_str!(
+    Ipv4Addr,
+    Ipv6Addr,
+    IpAddr,
+    SocketAddrV4,
+    SocketAddrV6,
+    SocketAddr,
+    PathBuf,
+    OsString
+);
+
 #[cfg(test)]
 mod tests {
     use crate::*;
@@ -628,10 +700,7 @@ mod tests {
     }
 
     impl FromEnvironment for Config {
-        fn from_env(
-            _: Option<Property<'_>>,
-            env: &mut SalakContext<'_>,
-        ) -> Result<Self, PropertyError> {
+        fn from_env(_: Option<Property<'_>>, env: &mut SalakContext<'_>) -> Res<Self> {
             Ok(Config {
                 i8: env.require_def("i8", None)?,
             })
